# 这些常量会在其他地方被用到，用作某些判断
import logging
import warnings

logger = logging.getLogger(__name__)

class ReversedStockStatus(object):
    # 默认状态
    INIT = "init"
    # 缺货
    SHORTAGE = "shortage"
    # 无货需要下架
    OFF = "off"
    # 备好货
    PREPARED = "prepared"


class ReversedStockType(object):
    PRODUCT = "product"
    GIFT = "gift"


class ReversedStockActions(object):
    # 导购标记货品
    GUIDE_MARK = "guide_mark"
    # 客服取货
    PICK_UP = "pick_up"
    # 客服还货
    RETURN = "return"


class ReversedStockNoteMessage(object):
    GUIDE_MARK = '标记货品状态为{0}'
    PICK_UP = '客服提货'
    RETURN = '客服还货'


class RequiredCountStock(object):
    IS_STOCK = "is_stock"
    IS_REQUIRED_COUNT = "is_required_count"
    IS_ANY = "is_any"

class AssignmentActions(object):
    # 创建配货
    CREATE = "create"
    # 关闭配货
    REMOVE = "remove"
    # 完成配货
    COMPLETE = "complete"
    # 已打印
    PRINTED = "printed"


class OrderActions(object):
    # 售后单独有一个Note表
    # 负库存相关单独有一个Note表
    # 修改
    MODIFY_ADDRESS = "modify_address"
    # 修改客服备注
    MODIFY_OPERATOR_NOTE = "modify_operator_note"
    # 关闭订单
    CLOSE = "close"
    # 发起支付
    PAY = "pay"
    # 完成支付订单
    COMPLETE_PAYMENT = "complete_payment"
    # 发货
    SEND = "send"
    # 用户提货
    TAKE = "take"
    # 其他
    OTHERS = "others"
    # 创建订单
    CREATE = "create"
    # 生成发货单
    GENERATE_DISTRIBUTE = "generate_distribute"
    # 给客户发短信
    SEND_SMS_MESSAGE = "send_sms_message"


class OrderTypes(object):
    # 线上订单
    ONLINE = "online"
    # 闪购
    FLASHSALE = "flashsale"
    # 贩卖机类型，现场直接出货
    VENDING = "vending"
    # 贩卖机类型，现场直接出货： 样机种草
    VENDING_MACHINE_HR = "vending_machine_hr"
    # 互动触摸屏类型，提货需要到专柜
    SCREEN = "screen"
    # rfid 订单类型
    RFID = "rfid"
    # 拼团订单
    GROUP = "group"
    # 积分商城
    POINTS_MALL = "points_mall"
    # 汉光贩售机
    HGVENDING = "hgvending"
    # 海淘
    HAITAO = "haitao"

    ORDER_TYPES_CHOICES = (
        (ONLINE, "线上订单"),
        (FLASHSALE, "闪购订单"),
        (VENDING, "售货机自提"),
        (VENDING_MACHINE_HR, "样机种草"),
        (SCREEN, "互动触摸屏"),
        (RFID, "RFID"),
        (GROUP, "拼团订单"),
        (HGVENDING, "汉光贩售机"),
        (HAITAO, "海淘"),
    )


class __OrderShippingMethods(object):
    warnings.warn("已废弃", DeprecationWarning)

    SELF_SERVICE = "self_service"
    EXPRESS = "express"

    def __getattribute__(self, name: str):
        value = super().__getattribute__(name)
        logger.warn("sparrow_order_lib.sparrow_order.constants.OrderShippingMethod 已废弃, 请使用 sparrow_order_lib.core.constants.ShippingMethod 替换")
        return value


OrderShippingMethods = __OrderShippingMethods()


class OrderStatus(object):
    """订单对外展示状态"""
    # 2018-02-08 确定新的四个统一订单对外状态
    # 待付款
    UNPAID = "unpaid"
    # 准备中
    PREPARING = "preparing"
    # 可提货，待发货
    TO_DELIVER = "to_deliver"
    # 已发货，已完成
    COMPLETED = "completed"
    # 已关闭/取消
    CLOSED = "closed"
    # 订单不再筛选售后状态！

    # 售后
    AFTERSALE = "after_sale"
    # 待发货
    TO_SHIP = "to_ship"
    # 可自提
    TO_PICKUP = "to_pickup"

    # 待付款
    # UNPAID = "unpaid"
    # # 备货中
    # IN_PREPARING = "in_preparing"
    # # 待发货
    # TO_SEND = "to_send"
    # # 待提货
    # TO_TAKE = "to_take"
    # # 部分发货
    # PARTIALLY_SENT = "partially_sent"
    # # 已发货
    # SENT = "sent"
    # # 部分提货
    # PARTIALLY_TAKEN = "partially_taken"
    # # 已提货
    # TAKEN = "taken"
    # # 退换/售后
    # IN_AFTERSALE = "in_aftersale"
    # # 已关闭/取消
    # CLOSED = "closed"


class OrderPayStatus(object):
    # 订单支付状态
    UNPAID = "unpaid"
    PAY_FINISHED = "pay_finished"
    CLOSED = "closed"
    # ------------- 分阶段支付 --------------
    # 定金待支付
    TO_DEPOSITPAY = "to_depositpay"
    # 尾款待支付
    TO_TAILPAY = "to_tailpay"
    # 尾款超时
    TAILPAY_OVERTIME = "tailpay_overtime"

    ORDER_PAY_STATUS_CHOICE = (
        (UNPAID, "未支付"),
        (PAY_FINISHED, "支付完成"),
        (CLOSED, "已关闭"),
        (TO_DEPOSITPAY, "待支付定金"),
        (TO_TAILPAY, "待支付尾款"),
        (TAILPAY_OVERTIME, "尾款超时"),
    )
    ORDER_PAY_STATUS_DICT = dict(ORDER_PAY_STATUS_CHOICE)


class OrderGroupStatus(object):
    # 拼团中
    PROGRESS = "progress"
    # 已成团
    SUCCESS = "success"
    # 拼团失败
    FAIL = "fail"


class OrderAssignStatus(object):
    '''订单配货状态'''
    # 无配货 - 没有ready或completed的发货单，或者说没有发货单或发货单只有closed的
    INIT = "init"
    # 部分配货 - 有ready的发货单，但是ready的发货单里的line的总数总量不全
    PARTIAL = "partial"
    # 全部配货 - 有不是closed的发货单，全部发货单里全部line数量总量和订单一致
    COMPLETED = "completed"


class OrderShippingStatus(object):
    '''订单发货状态'''
    # 未发货 - 没有completed的发货单
    INIT = "init"
    # 部分发货 - 有completed的发货单，里面的line总量不全
    PARTIAL = "partial"
    # 全部发货 - completed的发货单里面的line总量和订单一致
    COMPLETED = "completed"


class OrderAftersaleStatus(object):
    # 订单售后状态
    # 有待处理售后
    OPEN = "open"
    # 无售后
    NONE = "none"
    # 售后都完成（因为同时open的售后单只会有一单）
    DONE = "done"


class OrderExchangeStatus(object):
    '''### 订单换货状态###'''
    # 有待处理售后
    OPEN = "open"
    # 无售后
    NONE = "none"
    # 换货都完成（因为同时open的换货单只会有一单）
    DONE = "done"
    # 配货状态
    EXCHANGE_STATUS = (
        (OPEN, "有在进行的换货"),
        (NONE, "无换货"),
        (DONE, "换货已完成"),
    )

class CouponSource(object):
    # 线上发的券
    ONLINE = "online"
    # 店内发的券
    IN_STORE = "store"


class ProductSaleType(object):
    # 实物类型
    REAL = "REAL"

    # 虚拟商品
    VIRTUAL = "VIRTUAL"


class SettingsLabel(object):
    """
    系统配置
    """
    # 客服主管拦截
    SUPERVISOR_INTERCEPTION = "supervisor_interception"


class InnerApiUri(object):
    ALIPAY = "/api/sparrow_alipay/i/alipay_aftersale_returned/"
    REFUND_QUERY = "/api/sparrow_alipay/i/alipay_aftersale_returned_query/"
    PRODUCT = "/api/sparrow_products_i/pure_products/{}/"


class ErrorCode(object):
    SUCCESS = 0  # 成功
    FLASH_HAS_PICKUP = 201001  # 闪购订单已提货，不能发起退单。
    HAS_VALID_AFTERSALE = 201002  # 此订单已发起售后
    HAS_VALID_ASSIGNMENT = 201003  # 此订单已配货，请点击申请售后
    REFUND_ERROR = 201004  # 退款失败


class BatchAssignType(object):
    '''订单配货类型'''
    # 全部配货
    WHOLE = "whole"
    # 部分配货
    PARTIAL = "partial"
    # 按商品部分配货
    PRODUCT = "product"
    # 按专柜部分配货
    SHOP = "shop"


class BatchAssignTaskStatus(object):
    '''订单配货任务执行状态'''
    # 正在进行中
    IN_PROGRESS = "in_progress"
    # 已经完成
    COMPLETED = "completed"
    # 执行出错
    ERROR = "error"

class HGVendingStatus(object):
    # 初始态
    INIT = "init"
    # 已取货
    PICKED_UP = "picked_up"
    # 已售后
    REFUND = "refund"
    # 超时未支付关闭
    CLOSED = "closed"

    HGVENDING_STATUS_CHOICES = (
        (INIT, "初始态"),
        (PICKED_UP, "已取货"),
        (REFUND, "已售后"),
        (CLOSED, "已关闭")
    )


# 贩售机取货码过期检测定时任务运行间隔
HGVENDING_BULK_TASK_INTERVAL = 15 * 60


class DeliverTimeType(object):
    '''
    发货时间类型(fixed_time 固定时间/pay_relative_time 支付相对时间)
    '''
    FIXED_TIME = "fixed_time"
    PAY_RELATIVE_TIME = "pay_relative_time"
    DELIVERTIME_TYPE_CHOICES = (
        (FIXED_TIME, "固定时间"),
        (PAY_RELATIVE_TIME, "支付相对时间")
    )


class PayMethod(object):
    '''支付方法（once 一次支付/twice两次支付）'''
    ONCE = "once"
    TWICE = "twice"
    PAY_METHOD_CHOICES = (
        (ONCE, "一次支付"),
        (TWICE, "两次支付")
    )


class DeliverTimeStatus(object):
    '''发货时间配置状态(none无需配置/to_configure待配置/configured已配置)'''
    NONE = "none"
    TO_CONFIGURE = "to_configure"
    CONFIGURED = "configured"
    DELIVERTIME_STATUS_CHOICES = (
        (NONE, "无需配置"),
        (TO_CONFIGURE, "待配置"),
        (CONFIGURED, "已配置")
    )


class PayStepType(object):
    '''支付阶段类型(all全款/deposit定金/tailpay尾款)'''
    ONCE = "once"
    DEPOSIT = "deposit"
    TAIL = "tail"
    PAY_STEP_TYPE_CHOICES = (
        (ONCE, "全款"),
        (DEPOSIT, "定金"),
        (TAIL, "尾款")
    )


STEP_CHAR = "&"


class AssignmentStatus(object):
    '''发货单邮寄状态'''
    # 只要创建了发货单就认为是配货了
    # 已配货，待发货（无快递单号）或待提货
    READY = "ready"
    # 已完成-有快递单号了，或者用户来自提了
    COMPLETED = "completed"
    # 已取消
    # 发货单不能删除，只能关闭取消
    CLOSED = "closed"

class Order_Number_Pay_Status(object):
    # 子支付单状态(init初始/doing进行中/done已完成)
    INIT = "init"
    DOING = "doing"
    DONE = "done"
    FAILED = "failed"
    ORDER_NUMBER_PAY_STATUS_CHOICES = (
        (INIT, '初始'),
        (DOING, '进行中'),
        (DONE, "已完成"),
        (FAILED, "失败"),

    )
    ORDER_NUMBER_PAY_STATUS_DICT = dict(ORDER_NUMBER_PAY_STATUS_CHOICES)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-
"""desc: 项目中所使用的 日期、时间 通用方法

"""

import time
import datetime
import calendar
import pytz

TIME_ZONE = "Asia/Shanghai"  # TODO: 放入环境变量


class _date_time:
    __standard__ = 'ISO 8601:2004(E)'


# 年份格式
BASIC_YEAR = '%Y'
SHORT_YEAR = '%y'
# 月份格式
BASIC_MONTH = '%Y%m'
EXTEND_MONTH = '%Y-%m'
# 日期格式
BASIC_DATE = '%Y%m%d'
EXTEND_DATE = '%Y-%m-%d'
# BasicDateOriFmt = '%Y%j'  # 一年中的第几天
# ExtendDateOriFmt = '%Y-%j'  # 一年中的第几天
# BasicDateWeek = '%YW%U%w'  # 2020W013 2020 01 01 星期三  星期天作为每个星期的第一天   0-6
# ExtendDateWeek = '%Y-W%U-%w'  # 2020-W01-3
# 时间格式
EXTEND_TIME = '%H:%M:%S'
EXTEND_MICRO_TIME = '%H:%M:%S.%f'
# ExtendTimeZone = '%H:%M:%S%Z'
# ExtendTimeZoneMicro = '%H:%M:%S.%f%Z'
# 日期时间格式
EXTEND_DT = '%Y-%m-%d %H:%M:%S'  # 2020-01-01 01:01:01
EXTEND_T_DT = '%Y-%m-%dT%H:%M:%S'  # 2020-01-01T01:01:01
EXTEND_T_DT__ = '%Y-%m-%dT%H-%M-%S'  # 2020-01-01T01:01:01
EXTEND_MICRO_DT = '%Y-%m-%d %H:%M:%S.%f'  # 2020-01-01 01:01:01.000000
EXTEND_T_ZONE_DT = '%Y-%m-%dT%H:%M:%SZ'  # 2020-01-01T01:01:01Z
EXTEND_T_ZONE_MICRO_DT = '%Y-%m-%dT%H:%M:%S.%fZ'  # 2020-01-01T01:01:01.000000Z
EXTEND_DT_DAY_START = '%Y-%m-%d 00:00:00'
EXTEND_DT_DAY_END = '%Y-%m-%d 23:59:59'


TZ_LOCAL = pytz.timezone(TIME_ZONE)
TZ_UTC = pytz.timezone('UTC')
ONE_MICRO_SECONDS = datetime.timedelta(microseconds=1)
ONE_SECOND = datetime.timedelta(seconds=1)
ONE_HOUR = datetime.timedelta(hours=1)
ONE_DAY = datetime.timedelta(days=1)


class UtcUtil(object):
    """ 与 utc 相关的一些转化

    可将所有与时间转化的函数封装至此类
    """

    def __init__(self, time_zone=TIME_ZONE):
        """
        :param time_zone: 时区
        """
        self.time_zone = time_zone
        self.tz = pytz.timezone(self.time_zone)

    def dt2utcdt(self, dt, tzinfo=False):
        """ datetime -> datetime  本地时间-> UTC 时间

        :param dt: 本地时间  datetime.datetime
        :param tzinfo: 是否带时区信息  如果带，返回的是 2020-09-08 17:57:42.874000+00:00   如果不带  2020-09-08 17:57:42.874000
        """
        utc_dt = self.tz.localize(dt).astimezone(pytz.utc)
        if not tzinfo:
            utc_dt = utc_dt.replace(tzinfo=None)
        return utc_dt

    def utcdt2dt(self, utcdt, tzinfo=False):
        """ datetime -> datetime:  utc 时间转化为本地时间

        :param utcdt: utc 时间  datetime.datetime
        :param tzinfo: 是否带时区信息  如果带，返回的是 2020-09-08 17:57:42.874000+08:00   如果不带  2020-09-08 17:57:42.874000
        """
        dt = pytz.utc.localize(utcdt).astimezone(self.tz)
        if not tzinfo:
            dt = dt.replace(tzinfo=None)
        return dt

    def utcnow(self):
        return datetime.datetime.utcnow()

    def dt2timestamp(self, dt):
        """ 时间转对应的时间戳 """
        return time.mktime(dt.timetuple())

    def dt2timestampmicro(self, dt):
        """ 毫秒级时间戳 """
        timestamp = time.mktime(dt.timetuple())
        return int(timestamp * 1000)

    def dt2utctimestamp(self, dt):
        """ 本地时间转 utc 时间戳 """
        return self.dt2timestamp(self.dt2utcdt(dt))

    def dt_str_to_utc_str(self, dt_str):
        """ dt_str:  2020-01-01 08:00:00 格式的字符串
        ->  2020-01-01T00:00:00Z 格式的字符串
        """
        dt = datetime.datetime.strptime(dt_str, EXTEND_DT)
        utc_dt = self.dt2utcdt(dt)
        return utc_dt.strftime(EXTEND_T_ZONE_DT)

    def dt_str_to_utc_str2(self, dt_str):
        """ dt_str:  2020-01-01 08:00:00 格式的字符串
        ->  2020-01-01T00:00:00.000Z 格式的字符串
        """
        dt = datetime.datetime.strptime(dt_str, EXTEND_DT)
        utc_dt = self.dt2utcdt(dt)
        milli = int(round(utc_dt.microsecond / 1000, 0))
        suffix = '.%03dZ' % milli
        return utc_dt.strftime(EXTEND_T_DT) + suffix

    def utc_str_to_datetime(self, utc_str, utc_format=EXTEND_T_ZONE_DT):
        """ utc 字符串转为 datetime utc_str: """
        utc_dt = datetime.datetime.strptime(utc_str, utc_format)
        return utc_dt.replace(tzinfo=pytz.utc).astimezone(self.tz).replace(tzinfo=None)

    @staticmethod
    def get_date_month_start_and_end(date=None):
        if date is None:
            temp_date = datetime.datetime.now()
        else:
            temp_date = date

        start = datetime.datetime(temp_date.year, temp_date.month, 1)

        wday, days = calendar.monthrange(temp_date.year, temp_date.month)

        end = datetime.datetime(temp_date.year, temp_date.month, days)

        return start, end

    @staticmethod
    def get_every_month(start_time, end_time):
        date_list = []
        while start_time.year < end_time.year or (start_time.month <= end_time.month and start_time.year <= end_time.year):
            date_str = start_time.strftime("%Y-%m")
            date_list.append(date_str)
            start_time += datetime.timedelta(days=28)

        ret = []
        for d_str in date_list:
            if d_str not in ret:
                ret.append(d_str)
        return ret

    @staticmethod
    def get_every_day(start_time, end_time):
        date_list = []
        while start_time <= end_time:
            date_str = start_time.strftime("%Y-%m-%d")
            date_list.append(date_str)
            start_time += datetime.timedelta(days=1)
        return date_list

    @staticmethod
    def get_date_by_str(str=None, fmt=EXTEND_DATE):
        if str is None:
            return datetime.datetime.now()

        return datetime.datetime.strptime(str, fmt)

    def get_dt_with_tz(self, dt):
        """ 返回带有时区信息的时间格式

        如  2020-06-18 17:28:23  变为 2020-06-18 17:28:23+08:00

        :param dt:  datetime 格式，必须是与 self.time_zone 相同时区的时间
        :return:
        """
        return self.tz.localize(dt)

    def utc_with_tz(self, dt):
        """ 标准时间加上时间戳

        如 2020-06-18 00:00:00 -> 2020-06-18 00:00:00+00:00

        :param dt: datetime 格式，必须为 0 时区时间
        """
        return TZ_UTC.localize(dt)

    def winnt_timestamp_to_dt(self, ntstamp, local=True):
        """ 将 windows NT 时间戳转化为 datetime 类型

        windows nt 时间戳实际上是从 1602 年 1 月 1 日 UTC 时间至今的 100 ns 数
        :param ntstamp: 100 ns 数
        :param local: 本地时间还是 UTC 时间 True 返回本地时间 False 返回 UTC 时间
        :return: datetime.datetime 类型 的 UTC 时间
        """
        utc = datetime.datetime(1601, 1, 1) + \
            datetime.timedelta(microseconds=ntstamp//10)

        if local:
            return self.utcdt2dt(utc)

        return utc


class DatetimeUtil(object):
    """ 可封装一些 与 date 或 time 有关的方法 """

    def __init__(self):

        pass

    def datetime_to_weekday(self, source_date):
        """
        :param source_date: datetime类型-- 2020-08-01
        :return: str类型 : "周一"
        """
        week_dic = {0: "周一",
                    1: "周二",
                    2: "周三",
                    3: "周四",
                    4: "周五",
                    5: "周六",
                    6: "周日"
                    }
        week_num = source_date.weekday()
        return week_dic.get(week_num, None)

    @staticmethod
    def date_range(start, end, step=1):
        """ 返回开始时间结束时间内的日期。

        注意: 返回的日期列表包含 start 和 end

         :param start: 开始日期  datetime.date 或 'yyyy-mm-dd'  必须与 end 类型相同
         :param end: 结束日期  datetime.date 或 'yyyy-mm-dd'  必须与 start 类型相同
         :param step: 步长
        """
        is_str = isinstance(start, str)

        if is_str:
            start = datetime.datetime.strptime(start, EXTEND_DATE).date()
            end = datetime.datetime.strptime(end, EXTEND_DATE).date()

        result = [start+datetime.timedelta(days=delta)
                  for delta in range(0, (end-start).days+1, step)]
        if is_str:
            result = [day.strftime(EXTEND_DATE) for day in result]

        return result

    @staticmethod
    def yesterday():
        return datetime.datetime.today() - datetime.timedelta(days=1)

    def get_weekday_le_date(self, date, weekday_number):
        """
        返回 date 或date之前的第一个指定星期几的日期：类型为datetime
        :param date: datetime类型-- 2021-09-25
        :param weekday_number 想要设定的周几 int 类型 例如 周5就输入 4
                    {0: "周一",
                    1: "周二",
                    2: "周三",
                    3: "周四",
                    4: "周五",
                    5: "周六",
                    6: "周日" }
        :return: datetime类型 2021-09-24 (周五)
        """
        week_num = date.weekday()
        if week_num == weekday_number:
            return date
        else:
            if week_num > weekday_number:
                return date - datetime.timedelta(abs(weekday_number-week_num))
            else:
                return date - datetime.timedelta(days=abs(weekday_number - week_num + 5))

    @staticmethod
    def str2dt(dt_str):
        ''' 字符串转 datetime.datetime
        YYYY-mm-dd HH:MM:SS
        YYYY-mm-dd
        '''
        if len(dt_str) == 19:
            return datetime.datetime.strptime(dt_str, EXTEND_DT)
        if len(dt_str) == 12:
            return datetime.datetime.strptime(dt_str, EXTEND_DATE)
        # TODO: 完善其他格式


class IsHoliday(object):
    """ 节假日检测

     未解决国家农历法定节假日检测
    """

    @staticmethod
    def isHoliday(date_like):
        date_like = IsHoliday.get_datetime(date_like)
        w = date_like.strftime('%w')
        if int(w) in [0, 6]:
            return True
        elif (date_like.month, date_like.day) in IsHoliday.legal_holidays():
            return True
        else:
            return 0

    @staticmethod
    def get_datetime(date_like):
        if isinstance(date_like, datetime.date):
            return datetime.datetime.strptime(date_like.strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S')
        elif isinstance(date_like, datetime.datetime):
            return date_like
        elif isinstance(date_like, str):
            if date_like.count(':') == 2 and date_like.count('-') == 2 and len(date_like.strip()) == 19:
                return datetime.datetime.strptime(date_like, '%Y-%m-%d %H:%M:%S')
            elif date_like.count(':') == 0 and date_like.count('-') == 2 and len(date_like.strip()) == 10:
                return datetime.datetime.strptime(date_like + " 00:00:00", '%Y-%m-%d %H:%M:%S')
        raise ValueError(date_like)

    @staticmethod
    def legal_holidays():
        # 元组表示的月和日
        new_years_days = [(12, 30), (12, 31), (1, 1)]
        labor_days = [(4, 29), (4, 30), (5, 1)]
        national_days = [(10, d) for d in range(1, 8)]
        all_holidays = new_years_days + labor_days + national_days
        return all_holidays


def get_next_month(month_str):
    """获取下个月的字符串
    :param month_str: 如 2018-09
    :return: 字符串表示的月份: 如 2018-10
    """
    return get_month_later(month_str, 1)


def get_month_later(month_str, interval):
    """获取几个月之后

    :param month_str: 参考月
    :param interval: 间隔，正整数
    :return:
    """
    return get_month_by_interval(month_str, interval)


def get_month_earlier(month_str, interval):
    """获取几个月之前

    :param month_str: 参考月
    :param interval: 间隔，正整数
    :return:
    """
    return get_month_by_interval(month_str, -interval)


def get_month_by_interval(month_str, interval):
    """ 由间隔月份获取新月份 """
    if len(month_str) > 7:
        month_str = month_str[:7]
    year, month = [int(x) for x in month_str.split('-')]
    weight = year * 12 + month - 1
    weight_new = weight + interval
    return "%04d-%02d" % (weight_new // 12, weight_new % 12 + 1)


def get_month_list(start_month, end_month, border=True):
    """获取两个月份之间的所有月份列表

    border: 是否包含两端边界
    """
    month_lst = list()
    start_month, end_month = min(
        start_month, end_month), max(start_month, end_month)
    month = get_next_month(start_month)
    while month < end_month:
        month_lst.append(month)
        month = get_next_month(month)

    if border:
        month_lst = [start_month] + month_lst + [end_month]
    return month_lst


if __name__ == '__main__':

    pass



from rest_framework.exceptions import APIException
from rest_framework.status import HTTP_400_BAD_REQUEST

from ..sparrow_parcel.constants import PARCEL_ERROR_CODE_PREFIX


class ParcelException(APIException):
    status_code = HTTP_400_BAD_REQUEST
    default_detail = 'Bad_request'
    default_code = 'error'
    error_code = PARCEL_ERROR_CODE_PREFIX + 1


class ParcelRepeatScanException(ParcelException):
    error_code = PARCEL_ERROR_CODE_PREFIX + 2



from ..core.constants import UserRole

BASE_EXCHANGE_API_CODE = 20106000


class ExchangeOrderStatus(object):
    '''
    ### 换货单总单状态 ###
    '''
    IN_PROCESS = "in_process"
    CLOSED = "closed"
    DONE = "done"
    EXCHANGE_ORDER_STATUS_CHOICES = (
        (IN_PROCESS, "进行中"),
        (DONE, "已完成"),
        (CLOSED, "已关闭"),
    )


class ExchangeOrderStatusForDistribute:
    '''
    发货单上要展示的换货状态, 由 if_need_exchange 和 exchange_status 计算而来
    '''

    EXO_STATUS_MAP = {
        0: "无换货",
        1: "换货中",
        2: "换货完成",
    }


class ExchangeLineStatus(object):
    '''
    ### 换货单分单状态 ###
    '''
    # 待审核
    OPEN = "open"
    # 已关闭
    CLOSED = "closed"
    # 待客户退回商品
    PENDING_RETURN = "pending_return"
    # 待确收商品
    PENDING_CONFIRM = "pending_confirm"
    # 待换货
    PENDING_EXCHANGE = "pending_exchange"
    # 客服收到寄回商品
    CUSSERVICE_RECEIVED = "cusservice_received"
    # 退换货仓
    AFTERSALE_WH = "AftersaleWH"
    # 换货完成
    DONE = "done"

    EXCHANGE_LINE_STATUS_CHOICES = (
        (OPEN, "待审核"),
        (PENDING_RETURN, "待客人寄回"),
        (PENDING_CONFIRM, "待专柜确收"),
        (PENDING_EXCHANGE, "待寄出换货"),
        (CUSSERVICE_RECEIVED, "客服收到寄回商品"),
        (AFTERSALE_WH, "退货仓上架"),
        (DONE, "换货完成"),
        (CLOSED, "已关闭"),
    )
    EXCHANGE_LINE_STATUS_DICT = dict(EXCHANGE_LINE_STATUS_CHOICES)

    EXCHANGE_LINE_STATUS_CHOICES_FOR_A = EXCHANGE_LINE_STATUS_CHOICES

    EXCHANGE_LINE_STATUS_CHOICES_FOR_B = (
        (OPEN, "待审核"),
        (PENDING_RETURN, "待客人寄回"),
        (PENDING_CONFIRM, "待专柜确收"),
        (PENDING_EXCHANGE, "待寄出换货"),
        (CUSSERVICE_RECEIVED, "客服收到寄回商品"),
        (AFTERSALE_WH, "客服已收退货"),
        (DONE, "换货完成"),
        (CLOSED, "已关闭"),
    )

    EXCHANGE_LINE_STATUS_CHOICES_FOR_P = EXCHANGE_LINE_STATUS_CHOICES


class EventMapKey(object):

    TO_STATUS_KEY = "to_status"
    USER_ROLES_KEY = "user_roles"


class ExchangeEventConst(object):
    '''
    ### 事件 ###
    '''
    # 创建换货单
    NEW_ADD = "new_add"
    # 导购同意客户换货
    APPROVE = "approve"
    # 取消换货
    CANCEL = "cancel"
    # 客人自提
    CUS_SELF_SERVICE = "cus_self_service"
    # 客户回填运单信息
    CUS_WRITE_BACK = "cus_write_back"
    # 确收
    CONFIRM = "confirm"
    # 导购发货
    GUIDE_EXPRESS = "guide_express"
    # 留言
    COMMENT = "comment"
    # 打印
    PRINT = "print"
    # 修改换货地址
    MODIFY_ADDRESS = "modify_address"
    # 修改客户回填运单号
    MODIFY_CUS_SHIPPING_NUMBER = "modify_cus_shipping_number"
    # 催导购确收
    PUSH_GUIDE_CONFIRM = "push_guide_confirm"
    # 催导购审核
    PUSH_GUIDE_APPROVE = "push_guide_approve"
    # 催导购寄出
    PUSH_GUIDE_EXPRESS = "push_guide_express"
    # 客服收货
    CUSSERVICE_RECEIVE = "cusservice_receive"
    # 取消客服收货
    CANCEL_CUSSERVICE_RECEIVE = "cancel_cusservice_receive"
    # 售后仓入库、上架
    ENTER_AFTERSALE_WH = "enter_aftersale_wh"

    EVENT_2_BUTTONNAME_MAP = {
        NEW_ADD: "创建换货单",
        APPROVE: "通过",
        CANCEL: "关闭",
        CUS_SELF_SERVICE: "现换",
        CUS_WRITE_BACK: "客户回填运单信息",
        CONFIRM: "收换货",
        GUIDE_EXPRESS: "寄换货",
        COMMENT: "备注",
        PRINT: "打印",
        MODIFY_ADDRESS: "修改换货地址",
        MODIFY_CUS_SHIPPING_NUMBER: "修改客户回填运单号",
        PUSH_GUIDE_CONFIRM: "催确收",
        PUSH_GUIDE_APPROVE: "催审核",
        PUSH_GUIDE_EXPRESS: "催寄出",
        CUSSERVICE_RECEIVE: "客服收货",
        CANCEL_CUSSERVICE_RECEIVE: "取消客服收货",
        ENTER_AFTERSALE_WH: "售后仓入库",
    }

    EVENT_2_CNAME_MAP = {
        CUS_SELF_SERVICE: "客人现场换货",
        APPROVE: "通过审核",
        CANCEL: "关闭换货",
    }

    # 注意：
    # 1、当不存在EventMapKey.USER_ROLES_KEY的key时，说明所有角色都可以操作；
    # 2、当EventMapKey.TO_STATUS_KEY对应的value设置为None时，则说明动作不会造成状态变更；
    FROM_TO_MAP = {
        NEW_ADD: {  # 创建换货单
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSTOMER, UserRole.CUSSERVICE]
            }
        },
        APPROVE: {  # 导购同意客户换货
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.PENDING_RETURN,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            }
        },
        CANCEL: {  # 取消换货
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CLOSED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
        },
        CUS_SELF_SERVICE: {  # 客人自提
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.DONE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.DONE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.DONE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.DONE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
        },
        CUS_WRITE_BACK: {  # 客户回填运单信息
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.PENDING_CONFIRM,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSTOMER]
            },
        },
        CONFIRM: {  # 确收
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.PENDING_EXCHANGE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.PENDING_EXCHANGE,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
        },
        GUIDE_EXPRESS: {  # 导购发货
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.DONE,
                EventMapKey.USER_ROLES_KEY: [UserRole.GUIDE]
            },
        },
        PRINT: {  # 打印
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.DONE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
        },
        COMMENT: {
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.DONE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.GUIDE, UserRole.CUSSERVICE]
            },
        },
        MODIFY_ADDRESS: {
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        MODIFY_CUS_SHIPPING_NUMBER: {
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.CUSTOMER]
            },
        },
        PUSH_GUIDE_CONFIRM: {
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            },
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [UserRole.CUSSERVICE]
            }
        },
        PUSH_GUIDE_APPROVE: {
            ExchangeLineStatus.OPEN: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            },
        },
        PUSH_GUIDE_EXPRESS: {
            ExchangeLineStatus.PENDING_EXCHANGE: {
                EventMapKey.TO_STATUS_KEY: None,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            }
        },
        CUSSERVICE_RECEIVE: {
            ExchangeLineStatus.PENDING_RETURN: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CUSSERVICE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            },
            ExchangeLineStatus.PENDING_CONFIRM: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.CUSSERVICE_RECEIVED,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            },
        },
        CANCEL_CUSSERVICE_RECEIVE: {
            ExchangeLineStatus.CUSSERVICE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.PENDING_CONFIRM,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            },
        },
        ENTER_AFTERSALE_WH: {
            ExchangeLineStatus.CUSSERVICE_RECEIVED: {
                EventMapKey.TO_STATUS_KEY: ExchangeLineStatus.AFTERSALE_WH,
                EventMapKey.USER_ROLES_KEY: [
                    UserRole.CUSSERVICE, UserRole.AUTO]
            },
        },
    }

    # 按钮排序
    EVENT_CONST_SORT_MAP = {
        CANCEL: 0,
        COMMENT: 1,
        APPROVE: 2,
        CONFIRM: 2,
        GUIDE_EXPRESS: 2,
        PRINT: 3,
        CUS_SELF_SERVICE: 4,
    }


def get_available_event_list(exline_status, user_role):
    '''
    根据换货分单的状态和用户角色，获取可进行的操作事件列表
    '''
    available_event_list = []
    if not exline_status or not user_role:
        return available_event_list
    # special_event_list 记录了不需要展示按钮的事件
    special_event_list = []
    from_to_map = ExchangeEventConst.FROM_TO_MAP
    for event_code, event_dict in from_to_map.items():
        if event_code in special_event_list:
            continue

        if exline_status not in event_dict:
            continue
        to_status_dict = event_dict.get(exline_status)
        to_status_user_role_list = to_status_dict.get(
            EventMapKey.USER_ROLES_KEY, None)

        event_str = event_code + "|" + \
            ExchangeEventConst.EVENT_2_BUTTONNAME_MAP.get(event_code)

        # 当不存在EventMapKey.USER_ROLES_KEY的key时，说明所有角色都可以操作；
        if not to_status_user_role_list:
            available_event_list.append(event_str)
        else:
            if user_role in to_status_user_role_list:
                available_event_list.append(event_str)
    available_event_list.sort(
        key=lambda event_str: ExchangeEventConst.EVENT_CONST_SORT_MAP.get(event_str.split("|")[0], 0))
    return available_event_list


EXCHANGE_USER_REASON = [
    "无理由换货",
    "尺码/颜色不合适",
    "质量问题",
    "其他"
]


class ExchangeReturnShippingMethod:
    ''' 换货后客户取货方式 '''
    SELF_SERVICE = "self_service"
    EXPRESS = "express"
